@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 
 import numpy as np
-from numba import typed, types
+from numba import typed, types, prange
 
-from npbrain.core.neuron import Neurons
-from npbrain.core.synapse import Synapses
-from npbrain.utils import helper, profile
+from .neuron import Neurons
+from .synapse import Synapses
+from ..utils import helper, profile
 
 __all__ = [
     'Monitor',
@@ -42,29 +42,12 @@ class SpikeMonitor(Monitor):
     def __init__(self, target):
         # check `variables`
         self.vars = ('index', 'time')
+        num = target.state.shape[1]
 
         # check `target`
         assert isinstance(target, Neurons), 'Cannot monitor spikes in synapses.'
 
         # fake initialization
-        self.index = []
-        self.time = []
-        self.state = self.time
-        self.vars_idx = self.index
-
-        @helper.autojit('(f8[:, :], ListType(f8), ListType(i8), f8)')
-        def update_state(neu_state, mon_time, mon_index, t):
-            spike_idx = np.where(neu_state[-3] > 0.)[0]
-            for idx in spike_idx:
-                mon_index.append(idx)
-                mon_time.append(t)
-
-        self.update_state = update_state
-
-        # super class initialization
-        super(SpikeMonitor, self).__init__(target)
-
-    def init_state(self, length):
         if profile.is_numba_bk():
             self.index = typed.List.empty_list(types.int64)
             self.time = typed.List.empty_list(types.float64)
@@ -72,6 +55,24 @@ class SpikeMonitor(Monitor):
             self.index = []
             self.time = []
 
+        @helper.autojit
+        def update_state(neu_state, mon_time, mon_index, t):
+            for idx in prange(num):
+                if neu_state[-3, idx] > 0.:
+                    mon_index.append(idx)
+                    mon_time.append(t)
+
+        self.update_state = update_state
+
+        # super class initialization
+        super(SpikeMonitor, self).__init__(target)
+
+    def __str__(self):
+        return 'SpikeMonitor of {}'.format(str(self.target))
+
+    def __repr__(self):
+        return 'SpikeMonitor of {}'.format(repr(self.target))
+
 
 class StateMonitor(Monitor):
     """Monitor class to record states.
@@ -108,35 +109,28 @@ class StateMonitor(Monitor):
             setattr(self, k, np.zeros((1, 1)))
         self.state = []
 
-        # function of update state
-        @helper.autojit('(f8[:, :], UniTuple(f8[:, :], {}), i4[:], i4)'.format(len(vars)))
-        def record_neu_state(obj_state, mon_states, vars_idx, i):
-            for j, index in enumerate(vars_idx):
-                v = obj_state[index]
-                mon_states[j][i] = v
-
-        @helper.autojit('(UniTuple(f8[:, :], 3), UniTuple(f8[:, :], {}), i4[:, :], i4)'.format(len(vars)))
-        def record_syn_state(obj_state, mon_states, vars_idx, i):
-            var_len = len(vars_idx)
-            for j in range(var_len):
-                index = vars_idx[j]
-                v = obj_state[index[0]][index[1]]
-                mon_states[j][i] = v
-
-        # variable2index and update_state function
-        if isinstance(target, Neurons):
-            self.update_state = record_neu_state
-            var_idxs = np.array([target.var2index[v] for v in self.vars])
-            self.target_index_by_vars = lambda: var_idxs
-        elif isinstance(target, Synapses):
-            self.update_state = record_syn_state
-            if 'g_in' not in vars and 'g_out' not in vars:
-                var_idxs = np.array([target.var2index[v] for v in self.vars])
-                self.target_index_by_vars = lambda: var_idxs
+        if 'g_out' in vars or 'g_in' in vars:
+            if len([v for v in vars if v != 'g_out' and v != 'g_in']):
+                func_str = '''def func(obj_state, delay_state, mon_state, out_idx, in_idx, i):'''
             else:
-                self.target_index_by_vars = lambda: np.array([target.var2index[v] for v in self.vars])
+                func_str = '''def func(delay_state, mon_state, out_idx, in_idx, i):'''
         else:
-            raise ValueError('Unknown type.')
+            func_str = '''def func(obj_state, mon_state, i):'''
+        for j, k in enumerate(vars):
+            if k == 'g_out':
+                func_str += '\n\tmon_state[{}][i] = delay_state[out_idx]'.format(j)
+            elif k == 'g_in':
+                func_str += '\n\tmon_state[{}][i] = delay_state[in_idx]'.format(j)
+            else:
+                func_str += '\n\tmon_state[{}][i] = obj_state[{}]'.format(j, target.var2index[k])
+        exec(compile(func_str, '', 'exec'))
+
+        if profile.debug:
+            print('\nMonitor function:')
+            print('-' * 30)
+            print(func_str)
+
+        self.update_state = helper.autojit(locals()['func'])
 
         # super class initialization
         super(StateMonitor, self).__init__(target)
@@ -144,22 +138,24 @@ class StateMonitor(Monitor):
     def init_state(self, length):
         assert isinstance(length, int)
 
-        vars_idx = self.target_index_by_vars()
         mon_states = []
         for i, k in enumerate(self.vars):
-            index = vars_idx[i]
-            if isinstance(self.target, Synapses):
-                v = self.target.state
-                for idx in index:
-                    v = v[idx]
+            if k in ['g_out', 'g_in']:
+                v = self.target.delay_state[0]
             else:
-                v = self.target.state[index]
+                v = self.target.state[self.target.var2index[k]]
             shape = (length,) + v.shape
-            state = np.zeros(shape)
+            state = np.zeros(shape, dtype=profile.ftype)
             setattr(self, k, state)
             mon_states.append(state)
         self.state = tuple(mon_states)
 
+    def __str__(self):
+        return 'StateMonitor of {}'.format(str(self.target))
+
+    def __repr__(self):
+        return 'StateMonitor of {}'.format(repr(self.target))
+
 
 def raster_plot(mon, times=None):
     """Get spike raster plot which displays the spiking activity
