@@ -1,12 +1,14 @@
 # -*- coding: utf-8 -*-
-import numpy as np
-import numba as nb
 
-from npbrain.utils import helper, profile
+import numpy as np
+
+from ..utils import helper, profile
+from .neuron import _format_vars
 
 __all__ = [
     'format_delay',
-    'initial_syn_state',
+    'init_syn_state',
+    'init_delay_state',
     'Synapses',
 ]
 
@@ -38,69 +40,60 @@ def format_delay(delay, dt=None):
     return delay_len
 
 
-def initial_syn_state(delay,
-                      num_syn: int,
-                      num_syn_var: int = 0,
-                      num_post: int = 0,
-                      num_post_var: int = 0,
-                      num_pre: int = 0,
-                      num_pre_var: int = 0):
-    """For each state, it is composed by
-    (pre_shape_state, post_shape_state, syn_shape_state).
+def init_syn_state(num_syn, variables=None, parameters=None):
+    """Initialize the synapse state with (num_syn, ) shape.
 
     Parameters
     ----------
-    delay : float, int, None
-        The length of synapse delay.
-    num_pre : int
-        Number of neurons in pre-synaptic group.
-    num_post : int
-        Number of neurons in post-synaptic group.
     num_syn : int
-        Number of synapses.
-    num_pre_var : int
-        Number of variables with (num_pre, ) shape.
-    num_post_var : int
-        Number of variables with (num_post, ) shape.
-    num_syn_var : int
-        Number of variables with (num_syn, ) shape.
+        Number of the synapses.
+    variables : tuple, list, int
+        The variables of the neuron model.
+        Each variable has the shape of (num_syn, ).
+        If `variables` is an instance of `list` or `tuple`, each of them is
+        initialized as `zeros`.
+    parameters : dict
+        The parameter of the neuron models. Each of them can be modified in
+        the model running.
 
     Returns
     -------
-    state : tuple
-        Synapse state.
+    state : np.ndarray
+        The state of the synapse.
     """
+    if variables is None and parameters is None:
+        raise ValueError('variables and parameters cannot be both None.')
 
-    # state with (pre_num, ) shape #
-    ################################
-    # The state is:
-    #   pre_spike                 [[..........],
-    # --------------------------   [..........],
-    #   vars with num_pre shape    [..........],
-    # --------------------------   [..........]]
-    pre_shape_state = np.zeros((1 + num_pre_var, num_pre))
+    # get names and values of variables, parameters
+    var_names, var_values = _format_vars(variables, num_syn)
+    par_names, par_values = _format_vars(parameters, num_syn)
 
-    # state with (post_num, ) shape #
-    #################################
-    # The state is:
-    # ----------- [[..........],
-    # delays       [..........],
-    # -----------  [..........],
-    # other vars   [..........],
-    # -----------  [..........]]
+    # get state
+    names = var_names + par_names
+    values = var_values + par_values
 
+    state = np.zeros((len(names), num_syn), dtype=profile.ftype)
+    for i, v in enumerate(values):
+        state[i] = v
+
+    return state
+
+
+def init_delay_state(num_post, delay, variables=None, parameters=None):
     delay_len = format_delay(delay)
-    post_shape_state = np.zeros((delay_len + num_post_var, num_post))
 
-    # state with (num_syn, ) shape #
-    ################################
-    # The state is:
-    # -------------------------  [[..........],
-    #  vars with num_syn shape    [..........]
-    # -------------------------   [..........]]
-    syn_shape_state = np.zeros((num_syn_var, num_syn))
+    # get names and values of variables, parameters
+    var_names, var_values = _format_vars(variables, num_post)
+    par_names, par_values = _format_vars(parameters, num_post)
+
+    # get state
+    names = var_names + par_names
+    values = var_values + par_values
+
+    state = np.zeros((delay_len + len(names), num_post), dtype=profile.ftype)
+    for i, v in enumerate(values):
+        state[delay_len + i] = v
 
-    state = (syn_shape_state, post_shape_state, pre_shape_state)
     return state
 
 
@@ -119,25 +112,11 @@ class Synapses(object):
         for k, v in kwargs.items():
             setattr(self, k, v)
 
+        assert 'pre' in kwargs, 'Must define "pre" in synapses.'
+        assert 'post' in kwargs, 'Must define "post" in synapses.'
         self.post.pre_synapses.append(self)
         self.pre.post_synapses.append(self)
 
-        # check functions
-        assert 'update_state' in kwargs, 'Must provide "update_state" function.'
-        assert 'output_synapse' in kwargs, 'Must provide "output_synapse" function.'
-
-        wrapper = helper.autojit('(UniTuple(f8[:, :], 3), f8, i8, f8[:, :], f8[:, :])')
-        self.update_state = wrapper(self.update_state)
-
-        wrapper = helper.autojit('(UniTuple(f8[:, :], 3), i8, f8[:, :], f8[:, :])')
-        self.output_synapse = wrapper(self.output_synapse)
-
-        # check `name`
-        if 'name' not in kwargs:
-            global synapse_no
-            self.name = "Synapses-{}".format(synapse_no)
-            synapse_no += 1
-
         # check `num`, `num_pre` and `num_post`
         assert 'num' in kwargs, 'Must provide "num" attribute.'
         if 'num_pre' not in kwargs:
@@ -145,6 +124,19 @@ class Synapses(object):
         if 'num_post' not in kwargs:
             self.num_post = self.post.num
 
+        # check functions
+        assert 'update_state' in kwargs, 'Must provide "update_state" function.'
+        assert 'output_synapse' in kwargs, 'Must provide "output_synapse" function.'
+
+        self.update_state = helper.autojit(self.update_state)
+        self.output_synapse = helper.autojit(self.output_synapse)
+
+        # check `name`
+        if 'name' not in kwargs:
+            global synapse_no
+            self.name = "Synapses-{}".format(synapse_no)
+            synapse_no += 1
+
         # check `delay_len`
         if 'delay_len' not in kwargs:
             if 'delay' not in kwargs:
@@ -153,13 +145,19 @@ class Synapses(object):
                 dt = kwargs.get('dt', profile.get_dt())
                 self.delay_len = format_delay(self.delay, dt)
 
+        # check `state`
+        assert 'delay_state' in kwargs, 'Must define "delay_state" in synapses.'
+        if 'state' not in kwargs:
+            print('Synapses "{}" do not define "state" item.'.format(self.name))
+            self.state = None
+
         # check `var2index`
         if 'var2index' not in kwargs:
             raise ValueError('Must define "var2index".')
         assert isinstance(self.var2index, dict), '"var2index" must be a dict.'
         # "g_in" is the "delay_idx"
         # 'g_out' is the "output_idx"
-        default_var2index = {'pre_spike': (0, -1), 'g_in': [1, self.delay_len - 1], 'g_out': [1, 0]}
+        default_var2index = {'g_in': self.delay_len - 1, 'g_out': 0}
         self.default_var2index = default_var2index
         for k in default_var2index.keys():
             if k in self.var2index:
@@ -168,16 +166,16 @@ class Synapses(object):
         self.var2index.update(default_var2index)
 
     def update_conductance_index(self):
-        self.var2index['g_in'][1] = (self.var2index['g_in'][1] + 1) % self.delay_len
-        self.var2index['g_out'][1] = (self.var2index['g_out'][1] + 1) % self.delay_len
+        self.var2index['g_in'] = (self.var2index['g_in'] + 1) % self.delay_len
+        self.var2index['g_out'] = (self.var2index['g_out'] + 1) % self.delay_len
 
     @property
     def delay_idx(self):
-        return self.var2index['g_in'][1]
+        return self.var2index['g_in']
 
     @property
     def output_idx(self):
-        return self.var2index['g_out'][1]
+        return self.var2index['g_out']
 
     def __str__(self):
         return self.name
@@ -188,9 +186,3 @@ class Synapses(object):
     @property
     def available_monitors(self):
         return sorted(list(self.var2index.keys()))
-
-    def set_state(self, key, value):
-        if key not in self.var2index:
-            raise ValueError('Variable "{}" is not in the synapses.'.format(key))
-        i0, i1 = self.var2index[key]
-        self.state[i0][i1] = value
